@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2019 LOVE Development Team
+ * Copyright (c) 2006-2022 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -24,7 +24,9 @@
 #include <stdlib.h>
 
 #ifdef LOVE_WINDOWS
+#define WIN32_LEAN_AND_MEAN
 #include <malloc.h>
+#include <Windows.h>
 #else
 #include <unistd.h> // Assume POSIX support.
 #endif
@@ -54,8 +56,15 @@ void alignedFree(void *mem)
 size_t getPageSize()
 {
 #ifdef LOVE_WINDOWS
-	// TODO: Do an actual query.
-	return 4096;
+	static DWORD size = 0;
+	if (size == 0)
+	{
+		SYSTEM_INFO si;
+		GetSystemInfo(&si);
+		size = si.dwPageSize;
+	}
+
+	return (size_t) size;
 #else
 	static const long size = sysconf(_SC_PAGESIZE);
 	return size > 0 ? (size_t) size : 4096;
