@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2019 LOVE Development Team
+ * Copyright (c) 2006-2022 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -86,6 +86,9 @@ public:
 	int getMaxSourceEffects() const;
 	bool isEFXsupported() const;
 
+	void pauseContext();
+	void resumeContext();
+
 private:
 	float volume;
 	DistanceModel distanceModel;
