@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2019 LOVE Development Team
+ * Copyright (c) 2006-2022 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -70,8 +70,8 @@ private:
 	// A pointer to mapped memory.
 	char *memory_map;
 
-	size_t modified_offset;
-	size_t modified_size;
+	size_t modified_start;
+	size_t modified_end;
 
 }; // Buffer
 
