@@ -3,7 +3,7 @@ R"luastring"--(
 -- There is a matching delimiter at the bottom of the file.
 
 --[[
-Copyright (c) 2006-2019 LOVE Development Team
+Copyright (c) 2006-2020 LOVE Development Team
 
 This software is provided 'as-is', without any express or implied
 warranty.  In no event will the authors be held liable for any damages
@@ -368,7 +368,7 @@ local objectcache = setmetatable({}, {
 			width = width,
 			height = height,
 			format = format,
-			pointer = ffi.cast(conv.pointer, imagedata:getPointer()),
+			pointer = ffi.cast(conv.pointer, imagedata:getFFIPointer()),
 			tolua = conv.tolua,
 			fromlua = conv.fromlua,
 		}
